#!/usr/bin/python
# -*- encoding: utf-8 -*-

"""Author: 周鹏(chowroc)
Date: 2006-12-01
Email: chowroc.z@gmail.com

LFS useradd/groupadd only support lowercase,
and I need the '.' in the username for package classification

order:
groupadd --> useradd --> [_join(group, user)]
   |            |             |
/etc/group   /etc/passwd  /etc/group

If user/group exists, return the uid/gid


crablfs Copyright (c) 2006 周鹏(chowroc.z@gmail.com)

This file is part of crablfs.

crablfs is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

crablfs is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA"""

import re
import os
import pwd
import grp
import time
from crablfs import filetools

legal = '[^-\w\.]'
UID_MIN = 1000
UID_MAX = 20000
# 19999 < 20000, follows as the same
GID_MIN = 1000
GID_MAX = 20000
# for var in ['UID_MIN', 'UID_MAX', 'GID_MIN', 'GID_MAX']:
#	cmd = "grep 'PACK_%s' /etc/login.defs | sed -n '1p'" % var
#	tmp = os.popen(cmd).read().strip()
#	try:
#		n = int(re.split('[\t\s]+', tmp)[1])
#		exec "%s = %s" % (var, n)
#	except IndexError:
#		exec "%s = %s" % (var, var)
ADM_MIN = 950
ADM_MAX = 1000

def __user_compare(user1, user2):
	if user1[2] < user2[2]: return -1
	elif user1[2] > user2[2]: return 1
	else: return 0

def groupadd(group, type='normal'):
	if type == 'sysadm':
		gid_min = ADM_MIN
		gid_max = ADM_MAX
	else:
		gid_min = GID_MIN
		gid_max = GID_MAX
	if not group:
		raise "groupadd: Empty group name"
	mo = re.search(legal, group)
	if mo:
		raise "groupadd: Invalid group name '%s'" % group
	try:
		gid = grp.getgrnam(group)[2]
		if gid < gid_min:
			raise "groupadd: '%s' is not a package group" % group
		return gid
	except KeyError:
		pass
	# 如果组已存在，则直接返回 GID
	groups = grp.getgrall()
	groups.sort(__user_compare, reverse=True)
	# 按 GID 降序排列
	for i in range(len(groups)):
		if groups[i][2] < gid_max:
			max = groups[i][2]
			break
	if max < gid_min: gid = gid_min
	elif max + 1 == gid_max: raise "package groups' stack full."
	else: gid = max + 1
	file = open('/etc/group', 'a')
	file.write("%s:x:%s:\n" % (group, gid))
	file.close()
	gshadow(group)
	return gid

def useradd(user, group='', groups=[], home='', shell='/bin/bash'):
	if not user:
		raise "useradd: Empty user name"
	if type(groups) is not list:
		raise "useradd: Invalid groups '%s'" % groups
	if not home: home = '/home/%s' % user
	mo = re.search(legal, user)
	if mo:
		raise "useradd: Invalid user name '%s'" % user
	if not group: group = user
	gid = groupadd(group)
	try:
		uid = pwd.getpwnam(user)[2]
		if uid < UID_MIN:
			raise "useradd: '%s' is not a package user" % user
		### Need a _join() checking ! ###
		for G in groups: _join(G, user)
		return uid, gid
	except KeyError:
		users = pwd.getpwall()
		users.sort(__user_compare, reverse=True)
		for i in range(len(users)):
			if users[i][2] < UID_MAX:
				max = users[i][2]
				break
		if max < UID_MIN: uid = UID_MIN
		elif max + 1 == UID_MAX: raise "package users' stack full."
		else: uid = max + 1
		file = open('/etc/passwd', 'a')
		file.write("%s:x:%s:%s::%s:%s\n" % (user, uid, gid, home, shell))
		file.close()
		shadow(user)
		if not os.path.exists(home): os.mkdir(home)
		if os.path.isdir(home):
			os.chown(home, uid, gid)
		else:
			raise "useradd: HOME '%s' exists but not a directory" % home
		for G in groups: _join(G, user)
		return uid, gid

def _join(group, user):
	"Add user to a group"
	try:
		notex = 'group: %s' % group
		record = grp.getgrnam(group)
		notex = 'user: %s' % user
		pwd.getpwnam(user)
	except KeyError:
		raise "useradd: Can't join '%s' to '%s', '%s' not exists" % (user, group, notex)
	record = list(record)
	if user in record[3]: return
	# Have joined!
	oline = ','.join(record[3])
	users = oline
	record[3] = oline
	record[2] = str(record[2])
	oline = ':'.join(record)
	if users:
		rline = oline + ',' + user
	else:
		rline = oline + user
		# No ',' is needed for the first time, otherwise raise problems the next time
	# filetools.pull_list('/etc/group', oline)
	# filetools.push_list('/etc/group', rline)
	file = open('/etc/group', 'r+')
	L = file.readlines()
	i = L.index(oline + '\n')
	L[i] = rline + '\n'
	file.seek(0)
	file.truncate()
	file.writelines(L)
	file.close()

def shadow(user):
	days = int(time.time() / 86400)
	file = open('/etc/shadow', 'a')
	file.write("%s:!!:%s:0:99999:7:::\n" % (user, days))
	file.close()

def gshadow(group):
	file = open('/etc/gshadow', 'a')
	file.write("%s:!::" % group)
	file.close()



#!/usr/bin/python
# -*- encoding: utf-8 -*-

"""Author: 周鹏
Date: 2006-12-01
Email: chowroc.z@gmail.com

crablfs 的交互式命令解释器
主要在 userpack 的 install 时使用

需要一个变量赋值和读取功能(未实现)，如执行：
crablfs> cmd cp -Rv include/asm-$arch/* /usr/include/asm
比
crablfs> cmd cp -Rv include/asm-i386/* /usr/include/asm
要好

crablfs Copyright (c) 2006 周鹏(chowroc.z@gmail.com)

This file is part of crablfs.

crablfs is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

crablfs is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA"""

import cmd
from shlex import split
import os, sys

class cmdline(cmd.Cmd):
	def __init__(self, stdin=sys.stdin, stdout=sys.stdout):
		cmd.Cmd.__init__(self, 'tab', stdin, stdout)
		self.prompt = 'crablfs> '
		self.commands = []
		self.builtins = ['cd']
		self.stop = 0
		self.success = 0

	def build(self, command, argv):
		strerr = ""
		try:
			if command == 'cd':
				os.chdir(argv[1])
			else:
				strerr = "Invalid builtin command"
		except IndexError:
			strerr = "Invalid arguments for builtin command: %s" % command
		except OSError, (errno, errstr):
			strerr = "cmdline: %d, %s" % (errno, errstr)
		if strerr:
			sys.stderr.write("%s\n" % strerr)
			return 0
		return 1

	def do_command(self, args):
		"""execute the command the 'args' presents, 
and record the command into installation profile"""
		success = self.execute(args)
		if success:
			self.commands.append(args)
		else:
			strerr = "Invalid command"
			sys.stderr.write("%s\n" % strerr)
		return success
	do_cmd = do_command
	# *** 最好用 self.error_cmd_no 指向错误的命令数 ***
	# *** 如果这个命令没有被删除，则以失败退出？ ***

	# def complete(self, text, state):
	# 	"""Return the next possible completion for 'text'.

	# 	If a command has not been entered, then complete against command list.
	# 	Otherwise try to call complete_<command> to get list of completions.
	# 	"""
	# 	print "DEBUG: %s" % text
	# 	if state == 0:
	# 		import readline
	# 		origline = readline.get_line_buffer()
	# 		line = origline.lstrip()
	# 		stripped = len(origline) - len(line)
	# 		begidx = readline.get_begidx() - stripped
	# 		endidx = readline.get_endidx() - stripped
	# 		if begidx>0:
	# 			cmd, args, foo = self.parseline(line)
	# 			if cmd == '':
	# 				compfunc = self.completedefault
	# 			else:
	# 				try:
	# 					compfunc = getattr(self, 'complete_' + cmd)
	# 				except AttributeError:
	# 					compfunc = self.completedefault
	# 		else:
	# 			compfunc = self.completenames
	# 		self.completion_matches = compfunc(text, line, begidx, endidx)
	# 	try:
	# 		return self.completion_matches[state]
	# 	except IndexError:
	# 		return None

	def complete_command(self, text, line, begidx, endidx):
		import readline
		# import rlcompleter
		# readline.parse_and_bind("tab: complete")
		# readline.set_completer_delims(' \t\n`~!@#$%^&*()-=+[{]}\\|;:\'",<>;?') 
		readline.set_completer_delims(' \t\n`~!@#$%^&*()=+[{]}\\|;:\'",<>;?') 
		# kick out '-/' for file path name
		import glob
		return glob.glob('%s*' % text)
	complete_cmd = complete_command
	complete_do = complete_cmd

	def execute(self, args):
		"""Only execute the command 'args', and no recording"""
		success = 1
		try:
			try:
				argv = split(args)
				command = argv[0]
				if command in self.builtins:
					success = self.build(command, argv)
				else:
					# pid = os.fork()
					# if pid == 0:
					#	os.execvp(command, argv)
					#	success = 0
					# else: os.wait()
					success = (not os.system(args))
					# 使用 os.execvp() 将需要编写管道和重定向代码
			finally:
				return success
		except IndexError:
			strerr = "No command given"
			sys.stderr.write("%s\n" % strerr)
	do_do = execute
		
	def do_list(self, args):
		for i in range(len(self.commands)):
			print "%d, %s" % (i, self.commands[i])

	def do_delete(self, args):
		try:
			i = int(split(args)[0])
			print "delete: %d, %s" % (i, self.commands[i])
			self.commands.pop(i)
		except IndexError:
			strerr = "Index out of range, 'list' to show the commands"
			sys.stderr.write("%s\n" % strerr)
		except ValueError:
			strerr = "'list' to lookup the command's num that u want to del"
			sys.stderr.write("%s\n" % strerr)
	do_del = do_delete

	def do_commit(self, args): self.stop = 1; self.success = 1
	do_cmt = do_commit
	def do_rollback(self, args=""): self.commands = []
	def do_quit(self, args): self.do_rollback(); self.stop = 1
	def postcmd(self, stop, line): return self.stop
	# 只退出循环而不退出进程
	# 参见 cmd 源代码 cmdloop 和 postcmd 函数

# cli = cmdline()
# cli.cmdloop()
# print "after cmdloop"
# cli.do_command("ls")



#!/usr/bin/python
# -*- encoding: UTF-8 -*-

"""Author: 周鹏
Date: 2006-12-01
Email: chowroc.z@gmail.com

支持多种协议的文件传输模块

filetrans $SRC $DST
目前 $DST 必须存在且是目录
也没有模式匹配功能

crablfs Copyright (c) 2006 周鹏(chowroc.z@gmail.com)

This file is part of crablfs.

crablfs is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

crablfs is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Foobar; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA"""

import os
import re
import sys
import time
import getopt
import shutil
import ftplib

USER = os.environ['USER']
HOME = os.environ['HOME']

class filetrans:
	def __init__(self, user=USER, passwd='', host='localhost', 
	remote='', locale='.'):
		self.user = user
		self.host = host
		self.passwd = passwd
		self.remote = os.path.normpath(remote)
		self.locale = os.path.normpath(locale)
	def getfile(self, remote='', locale='', timeout=600, force=False):
		if remote: self.remote = os.path.normpath(remote)
		if locale: self.locale = os.path.normpath(locale)
		if not self.remote:
			strerr = "%s: No remote file given" % program
			sys.stderr.write("%s\n" % strerr)
		if not os.path.isdir(self.locale):
			strerr = "%s: No local directory %s found" % (program, self.locale)
			print >> sys.stderr, strerr
			sys.exit(1)
		# *** self.locale: 是普通文件？是目录？不存在？ ***
		# *** 或：是存在则当做目录，或不存在 ***
		# *** 不存在：self.rename ***
		# *** 如果这样应该返回目标串 ***
		
		fbname = os.path.basename(self.remote)
		destin = '%s/%s' % (self.locale, fbname)
		if os.path.exists(destin):
			if not force:
				strerr = "%s: Destination: %s exists" % (program, destin)
				print >> sys.stderr, strerr
				sys.exit(1)
			else:
				if os.path.isdir(destin):
					shutil.rmtree(destin)
				else:
					os.remove(destin)
		# scp 不会作出判断，所以在这里先判断
		self._getfile(timeout)
	def _getfile(self, timeout): pass

class filetransEx:
	def __init__(self, strerr):
		self.strerr = strerr
	def logerror(self):
		print self.strerr
		# sys.stderr.write("%s\n" % self.strerr)

class filetrans_copy(filetrans):
	def getfile(self, remote='', locale='', timeout=600, force=False):
		if remote: self.remote = os.path.normpath(remote)
		if locale: self.locale = os.path.normpath(locale)
		fbname = os.path.basename(self.remote)
		self.destin = os.path.join(self.locale, fbname)
		self.destin = os.path.abspath(self.destin)
		self.remote = os.path.abspath(self.remote)
		# Only the local coping can use 'abspath'!
		# if os.path.samefile(self.remote, self.destin):
		# 	return
		if self.remote == self.destin: # 如果是同一个文件
			return
		else:
			filetrans.getfile(self, timeout=timeout, force=force)
			# self._getfile(self, timeout=timeout, force=force)
	def _getfile(self, timeout):
		### print "DEBUG: filetrans_copy._getfile"
		if os.path.isdir(self.remote):
			shutil.copytree(self.remote, self.destin)
		else:
			shutil.copy(self.remote, self.locale)
filetrans_file = filetrans_copy

class filetrans_ftp(filetrans):
	def __init__(self, user='anoymous', passwd='', host='localhost',
		remote='', locale='.'):
		filetrans.__init__(self, user, passwd, host, remote, locale)
	def _getfile(self, timeout):
		self.__ftp_get(self.remote, self.locale)
	def __ftp_get(self, rdir, ldir):
		ftp = ftplib.FTP(self.host, self.user, self.passwd)
		ftp.login()
		fbname = os.path.basename(rdir)
		for sub in ftp.nlst(rdir):
			if sub == rdir:
			# is regular file
				rfile = sub
				fbname = os.path.basename(sub)
				lfile_o = open('%s/%s' % (ldir, fbname), 'wb')
				ftp.retrbinary('RETR %s' % rfile, lfile_o.write)
				lfile_o.close()
			else:
			# is directory
				ldir = '%s/%s' % (ldir, fbname)
				os.mkdir(ldir)
				rdir = sub
				self.__ftp_get(rdir, ldir)

class filetrans_ssh_pexpect(filetrans):
	def _getfile(self, timeout):
		cmd = 'scp -rpC %s@%s:%s %s/' % (self.user, self.host, self.remote, self.locale)
		# *** 如果未拷贝到文件，如远端文件根本不存在，这里没有抛出异常？ ***
		self.__pexpect(cmd, timeout)

	def __pexpect(self, cmd, timeout=600):
		import pexpect
		T = time.time()
		try:
			ssh = pexpect.spawn(cmd, timeout=timeout)
			index = -1
			while index < 2:
				index = ssh.expect([
					'continue connecting', 
					'password: ', 
					pexpect.EOF, 
					pexpect.TIMEOUT	])
				if index == 0:
					ssh.sendline('yes')
				elif index == 1:
					ssh.sendline(self.passwd)
		except pexpect.EOF:
			ssh.close()
			outstr = 'filetrans takes %d seconds' % int(time.time() - T)
			print outstr
		except pexpect.TIMEOUT:
		# *** 无法捕捉到这个异常？！ ***
			ssh.close()
			raise filetransEx('%s: pexpect TIMEOUT %d seconds' % (program, timeout))
		except ExceptionPexpect:
			raise filetransEx('filetrans_scp_pexpect failed, "%s"' % cmd)
		ssh.expect(pexpect.EOF)
		ssh.close()
		outstr =  'filetrans: [[32m%s[00m] takes %d seconds' % (cmd, int(time.time() - T))
		print outstr
filetrans_scp= filetrans_ssh_pexpect

def parse(remote):
	# pregexp = r"((?P<proto>\w+)://){0,1}(?P<host>(\w+\.{0,1})+){0,1}(?P<file>/.+){1}"
	pregexp = r"((?P<proto>\w+)://((?P<user>[\w\.]+)@'(?P<passwd>.+)':){0,1}){0,1}(?P<host>(\w+\.{0,1})+/){0,1}(?P<file>/{0,1}.*){1}"
	try:
		mo = re.match(pregexp, remote)
		proto = mo.group('proto')
		user = mo.group('user')
		passwd = mo.group('passwd')
		host = mo.group('host')
		fname = mo.group('file')
		return proto, user, passwd, host, fname
	except AttributeError:
		strerr = "%s: Not a valid file location: %s" % (program, remote)
		sys.stderr.write("%s\n" % strerr)
		sys.exit(1)

def build(user='', passwd='', remote='', locale='.'):
	proto, utmp, ptmp, host, remote = parse(remote)
	if utmp: user = utmp
	if ptmp: passwd = ptmp
	import filetrans
	try:
		ftclass = getattr(filetrans, 'filetrans_%s' % proto, filetrans_copy)
	except AttributeError:
		strerr = "%s: Not a valid protocol: %s" % (program, proto)
		sys.stderr.write("%s\n" % strerr)
		sys.exit(1)
	fto = ftclass(user, passwd, host, remote, locale)
	return fto

if __name__ == '__main__':
	program = os.path.basename(sys.argv[0])
	location = os.path.dirname(sys.argv[0])
	usage = """usage: filetrans [OPTIONS] $SOURCE $DESTINATION
	-u $user
	-p $password
	-t $timeout
	-f, force
	-G, GETFILE(default)
	-P, PUTFILE
	-h"""
	try:
		opts, args = getopt.getopt(sys.argv[1:], 'u:p:t:fG:P:', 
			['user=', 'password=', 'timeout', 'force', 'get', 'put', 'help'])
		user = ''
		passwd = ''
		source = args[0]
		destin = args[1]
		timeout = 600
		action = 'getfile'
		remote = source
		locale = destin
		force = False
		for o, v in opts:
			if o in ['-u', '--user']:
				user = v
			elif o in ['-p', '--password']:
				passwd = v
			elif o in ['-t', '--timeout']:
				timeout = int(v)
			elif o in ['-f', '--force']:
				force = True
			elif o in ['-D', '--get']:
				action = 'getfile'
				remote = source
				locale = destin
			elif o in ['-U', '--put']:
				action = 'putfile'
				remote = destin
				locale = source
		fto = build(user, passwd, remote, locale)
		exec "fto.%s(timeout=%s, force=%s)" % (action, timeout, force)
	except getopt.GetoptError, (errno, errstr):
		strerr = "%s: getopt error: %s, %s" % (program, errno, errstr)
		sys.stderr.write("%s\n", strerr)
		print usage
		sys.exit(1)
	except IndexError, errstr:
		strerr = "%s: Lack of source or destination: %s" % (program, errstr)
		sys.stderr.write("%s\n" % strerr)
		print usage
		sys.exit(1)
	# except filetransEx, ftEx:
	#	ftEx.logerror()
else:
	program = __name__



#!/usr/bin/python
# -*- encoding: utf-8 -*-

# Author: ÖÜÅô
# Date: 2006-10-02
# Email: chowroc.z@gmail.com

from distutils.core import setup

setup(
	name = 'crablfs',
	version = '0.1.1beta',
	author = 'Zhou Peng(chowroc)',
	author_email = 'chowroc.z@gmail.com',
	description = "User Based Package Management System",
	platforms = 'platform independent',
	license = 'GPL version 2',
	# classifier = [
	#	'Environment :: Console',
	#	'Operating System :: POSIX',
	#	'Programming Language :: Python',
	#	'Topic :: Communications :: Email',
	# ],
	py_modules = ['userpack'],
	package_dir = {'crablfs' : 'lib'},
	packages = ['crablfs'],
	scripts = ['userpack', 'crablfs', 'scripts/copy-profiles'],
	data_files = [('/etc', ['userpack.dirs'])],
)

# from distutils import file_util
# file_util.copy_file('userpack', 'upm', link='sym')



#!/usr/bin/python
# -*- encoding: utf-8 -*-

# Author: 周鹏
# Date: 2006-12-01
# Email: chowroc.z@gmail.com

import os
import sys
import shutil

def find_list(filename, inew):
	try:
		if type(inew) is not str:
			strerr = "%s is not a valid LIST item string" % inew
			print >> sys.stderr, strerr
			raise "find_list_error"
		file = open(filename, 'r')
		for item in file.readlines():
			item = item.strip()
			if inew == item: return
		raise "find_list_error"
	except Exception, (errno, errstr):
		strerr = "find list faild: %d, %s" % (errno, errstr)
		print >> sys.stderr, strerr
		raise "find_list_error"

def push_list(filename, inew):
	"如果一个条目不在列表文件中，则置入"
	try:
		if type(inew) is not str:
			strerr = "%s is not a valid LIST item string" % inew
			sys.stderr.write("%s\n" % strerr)
			return
		open(filename, 'a')  # touch $filename
		if os.path.isfile(filename): mode = 'r+'
		# else: mode = 'w'
		file = open(filename, mode)
		for item in file.readlines():
			item = item.strip()
			if inew == item: return
			# 如果已经存在则什么也不做
		file.write("%s\n" % inew)
		file.close()
	except Exception, (errno, errstr):
		strerr = "push list faild: %d, %s" % (errno, errstr)
		sys.stderr.write("%s\n" % strerr)

def pull_list(filename, inew):
	"从列表文件中删除一个条目"
	try:
		if type(inew) is not str:
			strerr = "%s is not a valid LIST item string" % inew
			sys.stderr.write("%s\n" % strerr)
			raise "pull_list_error"
		file = open(filename, 'r+')
		items = file.readlines()
		i = -1
		for item in items:
			itmp = item.strip()
			if inew == itmp:
				i = items.index(item)
				items.pop(i)
				break
		if i < 0:
			strerr = "LIST item: %s was not found" % inew
			sys.stderr.write("%s\n" % strerr)
			raise "pull_list_error"
		else:
			file.seek(0)
			file.truncate()
			file.writelines(items)
		file.close()
	except Exception, (errno, errstr):
		strerr = "pull list faild: %d, %s" % (errno, errstr)
		sys.stderr.write("%s\n" % strerr)
		raise "pull_list_error"

def rmtree(pathname):
	if os.path.isdir(pathname):
		shutil.rmtree(pathname)
	else:
		os.remove(pathname)

def copytree(pathname):
	if os.path.isdir(pathname):
		shutil.copytree(pathname)
	else:
		shutil.copy(pathname)



#!/usr/bin/python
# -*- encoding: utf-8 -*-

# Author: 周鹏
# Date: 2006-12-01
# Email: chowroc.z@gmail.com

"""为 userpack 解析配置
在将来可用 coreea 的 plainconf 替换"""

import re
import sys

def getconf(confile, flag='norm', *keys):
	options = {}
	found = []
	if type(confile) is file:
		confile = confile
	elif type(confile) is str:
		confile = open(confile, 'r')
	for ostr in confile.readlines():
		try:
			mo = re.match(
				'^\s*(?P<key>\w+)\s*=\s*"(?P<value>.*)";\s*$', ostr)
			key = mo.group('key')
			if key not in keys: continue
			value = mo.group('value')
			if flag == 'seq':
				if not options.get(key): options[key] = [value]
				else: options[key].append(value)
			else:
				options[key] = value
			found.append(key)
		except AttributeError:
			strerr = 'plainconf: %s is not a valid option' % ostr
			print >> sys.stderr, strerr
	for key in keys:
		if key not in found:
			strerr = 'plainconf: key %s was not found' % key
			print >> sys.stderr, strerr
	return options

def setconf(confile, mode, **options):
	if type(confile) is file:
		confile = confile
	elif type(confile) is str:
		confile = open(confile, mode)
	for key, value in options.iteritems():
		confile.write('%s = "%s";\n' % (key, value))
	# confile.close()

# optmap = getconf('/usr/src/rxvt/.config', 'norm', 'user', 'notexists')
# print optmap['notexists']
